@@ -14,23 +14,23 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 /**
  * SECTION:element-fpsdisplaysink
+ * @title: fpsdisplaysink
  *
  * Can display the current and average framerate as a testoverlay or on stdout.
  *
- * <refsect2>
- * <title>Example launch lines</title>
+ * ## Example launch lines
  * |[
- * gst-launch videotestsrc ! fpsdisplaysink
- * gst-launch videotestsrc ! fpsdisplaysink text-overlay=false
- * gst-launch filesrc location=video.avi ! decodebin2 name=d ! queue ! fpsdisplaysink d. ! queue ! fakesink sync=true
- * gst-launch playbin2 uri=file:///path/to/video.avi video-sink="fpsdisplaysink" audio-sink=fakesink
+ * gst-launch-1.0 videotestsrc ! fpsdisplaysink
+ * gst-launch-1.0 videotestsrc ! fpsdisplaysink text-overlay=false
+ * gst-launch-1.0 filesrc location=video.avi ! decodebin name=d ! queue ! fpsdisplaysink d. ! queue ! fakesink sync=true
+ * gst-launch-1.0 playbin uri=file:///path/to/video.avi video-sink="fpsdisplaysink" audio-sink=fakesink
  * ]|
- * </refsect2>
+ *
  */
 /* FIXME:
  * - can we avoid plugging the textoverlay?
@@ -47,9 +47,7 @@
 #include "config.h"
 #endif
 
-#include "debugutils-marshal.h"
 #include "fpsdisplaysink.h"
-#include <gst/interfaces/xoverlay.h>
 
 #define DEFAULT_SIGNAL_FPS_MEASUREMENTS FALSE
 #define DEFAULT_FPS_UPDATE_INTERVAL_MS 500      /* 500 ms */
@@ -102,6 +100,8 @@ static void fps_display_sink_set_property (GObject * object, guint prop_id,
 static void fps_display_sink_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 static void fps_display_sink_dispose (GObject * object);
+static void fps_display_sink_handle_message (GstBin * bin,
+    GstMessage * message);
 
 static gboolean display_current_fps (gpointer data);
 
@@ -114,6 +114,7 @@ fps_display_sink_class_init (GstFPSDisplaySinkClass * klass)
 {
   GObjectClass *gobject_klass = G_OBJECT_CLASS (klass);
   GstElementClass *gstelement_klass = GST_ELEMENT_CLASS (klass);
+  GstBinClass *bin_class = GST_BIN_CLASS (klass);
 
   parent_class = g_type_class_peek_parent (klass);
 
@@ -121,6 +122,8 @@ fps_display_sink_class_init (GstFPSDisplaySinkClass * klass)
   gobject_klass->get_property = fps_display_sink_get_property;
   gobject_klass->dispose = fps_display_sink_dispose;
 
+  bin_class->handle_message = fps_display_sink_handle_message;
+
   g_object_class_install_property (gobject_klass, PROP_SYNC,
       g_param_spec_boolean ("sync",
           "Sync", "Sync on the clock (if the internally used sink doesn't "
@@ -194,13 +197,10 @@ fps_display_sink_class_init (GstFPSDisplaySinkClass * klass)
    * @avgfps: The average fps
    *
    * Signals the application about the measured fps
-   *
-   * Since: 0.10.20
    */
   fpsdisplaysink_signals[SIGNAL_FPS_MEASUREMENTS] =
       g_signal_new ("fps-measurements", G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_LAST, 0, NULL, NULL,
-      __gst_debugutils_marshal_VOID__DOUBLE_DOUBLE_DOUBLE,
+      G_SIGNAL_RUN_LAST, 0, NULL, NULL, NULL,
       G_TYPE_NONE, 3, G_TYPE_DOUBLE, G_TYPE_DOUBLE, G_TYPE_DOUBLE);
 
   gstelement_klass->change_state = fps_display_sink_change_state;
@@ -208,62 +208,38 @@ fps_display_sink_class_init (GstFPSDisplaySinkClass * klass)
   gst_element_class_add_static_pad_template (gstelement_klass,
       &fps_display_sink_template);
 
-  gst_element_class_set_details_simple (gstelement_klass,
+  gst_element_class_set_static_metadata (gstelement_klass,
       "Measure and show framerate on videosink", "Sink/Video",
       "Shows the current frame-rate and drop-rate of the videosink as overlay or text on stdout",
       "Zeeshan Ali <zeeshan.ali@nokia.com>, Stefan Kost <stefan.kost@nokia.com>");
 }
 
-static gboolean
-on_video_sink_data_flow (GstPad * pad, GstMiniObject * mini_obj,
+static GstPadProbeReturn
+on_video_sink_data_flow (GstPad * pad, GstPadProbeInfo * info,
     gpointer user_data)
 {
+  GstMiniObject *mini_obj = GST_PAD_PROBE_INFO_DATA (info);
   GstFPSDisplaySink *self = GST_FPS_DISPLAY_SINK (user_data);
 
-#if 0
   if (GST_IS_BUFFER (mini_obj)) {
-    GstBuffer *buf = GST_BUFFER_CAST (mini_obj);
+    GstClockTime ts;
 
-    if (GST_CLOCK_TIME_IS_VALID (self->next_ts)) {
-      if (GST_BUFFER_TIMESTAMP (buf) <= self->next_ts) {
-        self->frames_rendered++;
-      } else {
-        GST_WARNING_OBJECT (self, "dropping frame : ts %" GST_TIME_FORMAT
-            " < expected_ts %" GST_TIME_FORMAT,
-            GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
-            GST_TIME_ARGS (self->next_ts));
-        self->frames_dropped++;
-      }
-    } else {
-      self->frames_rendered++;
+    /* assume the frame is going to be rendered. If it isnt', we'll get a qos
+     * message and reset ->frames_rendered from there.
+     */
+    g_atomic_int_inc (&self->frames_rendered);
+
+    ts = gst_util_get_timestamp ();
+    if (G_UNLIKELY (!GST_CLOCK_TIME_IS_VALID (self->start_ts))) {
+      self->interval_ts = self->last_ts = self->start_ts = ts;
     }
-  } else
-#endif
-  if (GST_IS_EVENT (mini_obj)) {
-    GstEvent *ev = GST_EVENT_CAST (mini_obj);
-
-    if (GST_EVENT_TYPE (ev) == GST_EVENT_QOS) {
-      GstClockTimeDiff diff;
-      GstClockTime ts;
-
-      gst_event_parse_qos (ev, NULL, &diff, &ts);
-      if (diff <= 0.0) {
-        g_atomic_int_inc (&self->frames_rendered);
-      } else {
-        g_atomic_int_inc (&self->frames_dropped);
-      }
-
-      ts = gst_util_get_timestamp ();
-      if (G_UNLIKELY (!GST_CLOCK_TIME_IS_VALID (self->start_ts))) {
-        self->interval_ts = self->last_ts = self->start_ts = ts;
-      }
-      if (GST_CLOCK_DIFF (self->interval_ts, ts) > self->fps_update_interval) {
-        display_current_fps (self);
-        self->interval_ts = ts;
-      }
+    if (GST_CLOCK_DIFF (self->interval_ts, ts) > self->fps_update_interval) {
+      display_current_fps (self);
+      self->interval_ts = ts;
     }
   }
-  return TRUE;
+
+  return GST_PAD_PROBE_OK;
 }
 
 static void
@@ -278,6 +254,14 @@ update_sub_sync (GstElement * sink, gpointer data)
     GST_WARNING ("Internal sink doesn't have sync property");
 }
 
+static void
+update_sub_sync_foreach (const GValue * item, gpointer data)
+{
+  GstElement *sink = g_value_get_object (item);
+
+  update_sub_sync (sink, data);
+}
+
 static void
 fps_display_sink_update_sink_sync (GstFPSDisplaySink * self)
 {
@@ -288,7 +272,8 @@ fps_display_sink_update_sink_sync (GstFPSDisplaySink * self)
 
   if (GST_IS_BIN (self->video_sink)) {
     iterator = gst_bin_iterate_sinks (GST_BIN (self->video_sink));
-    gst_iterator_foreach (iterator, (GFunc) update_sub_sync,
+    gst_iterator_foreach (iterator,
+        (GstIteratorForeachFunction) update_sub_sync_foreach,
         (void *) &self->sync);
     gst_iterator_free (iterator);
   } else
@@ -305,7 +290,7 @@ update_video_sink (GstFPSDisplaySink * self, GstElement * video_sink)
 
     /* remove pad probe */
     sink_pad = gst_element_get_static_pad (self->video_sink, "sink");
-    gst_pad_remove_data_probe (sink_pad, self->data_probe_id);
+    gst_pad_remove_probe (sink_pad, self->data_probe_id);
     gst_object_unref (sink_pad);
     self->data_probe_id = -1;
 
@@ -332,8 +317,9 @@ update_video_sink (GstFPSDisplaySink * self, GstElement * video_sink)
 
   /* attach or pad probe */
   sink_pad = gst_element_get_static_pad (self->video_sink, "sink");
-  self->data_probe_id = gst_pad_add_data_probe (sink_pad,
-      G_CALLBACK (on_video_sink_data_flow), (gpointer) self);
+  self->data_probe_id = gst_pad_add_probe (sink_pad,
+      GST_PAD_PROBE_TYPE_DATA_BOTH, on_video_sink_data_flow,
+      (gpointer) self, NULL);
   gst_object_unref (sink_pad);
 }
 
@@ -425,11 +411,7 @@ display_current_fps (gpointer data)
     g_free (self->last_message);
     self->last_message = g_strdup (fps_message);
     GST_OBJECT_UNLOCK (self);
-#if !GLIB_CHECK_VERSION(2,26,0)
-    g_object_notify ((GObject *) self, "last-message");
-#else
     g_object_notify_by_pspec ((GObject *) self, pspec_last_message);
-#endif
   }
 
   self->last_frames_rendered = frames_rendered;
@@ -511,11 +493,7 @@ fps_display_sink_stop (GstFPSDisplaySink * self)
     g_free (self->last_message);
     self->last_message = str;
     GST_OBJECT_UNLOCK (self);
-#if !GLIB_CHECK_VERSION(2,26,0)
-    g_object_notify ((GObject *) self, "last-message");
-#else
     g_object_notify_by_pspec ((GObject *) self, pspec_last_message);
-#endif
   }
 
   GST_OBJECT_LOCK (self);
@@ -696,6 +674,28 @@ fps_display_sink_change_state (GstElement * element, GstStateChange transition)
   return ret;
 }
 
+static void
+fps_display_sink_handle_message (GstBin * bin, GstMessage * message)
+{
+  GstFPSDisplaySink *self = (GstFPSDisplaySink *) bin;
+
+  if (GST_MESSAGE_TYPE (message) == GST_MESSAGE_QOS) {
+    GstFormat format;
+    guint64 rendered, dropped;
+
+    gst_message_parse_qos_stats (message, &format, &rendered, &dropped);
+    if (format != GST_FORMAT_UNDEFINED) {
+      if (rendered != -1)
+        g_atomic_int_set (&self->frames_rendered, rendered);
+
+      if (dropped != -1)
+        g_atomic_int_set (&self->frames_dropped, dropped);
+    }
+  }
+
+  GST_BIN_CLASS (parent_class)->handle_message (bin, message);
+}
+
 GType
 fps_display_sink_get_type (void)
 {
